@@ -49,12 +49,14 @@ struct _GdictSpellerPrivate
   gchar *word;
 
   GtkWidget *treeview;
-  GtkWidget *close;
+  GtkWidget *clear_button;
 
   GdkCursor *busy_cursor;
 
   GtkListStore *store;
   gint results;
+
+  GtkTooltips *tips;
   
   guint start_id;
   guint end_id;
@@ -94,7 +96,6 @@ enum
 enum
 {
   WORD_ACTIVATED,
-  CLOSED,
 
   LAST_SIGNAL
 };
@@ -145,7 +146,7 @@ set_gdict_context (GdictSpeller *speller,
 
   if (!GDICT_IS_CONTEXT (context))
     {
-      g_warning ("Object of type '%s' instead of a GdictContext\n",
+      g_warning ("Object of type `%s' instead of a GdictContext\n",
       		 g_type_name (G_OBJECT_TYPE (context)));
       return;
     }
@@ -168,9 +169,15 @@ gdict_speller_finalize (GObject *gobject)
   if (priv->busy_cursor)
     gdk_cursor_unref (priv->busy_cursor);
 
+  if (priv->tips)
+    g_object_unref (priv->tips);
+
   g_free (priv->strategy);
   g_free (priv->database);
   g_free (priv->word);
+
+  if (priv->store)
+    g_object_unref (priv->store);
     
   G_OBJECT_CLASS (gdict_speller_parent_class)->finalize (gobject);
 }
@@ -223,6 +230,8 @@ gdict_speller_get_property (GObject    *gobject,
     case PROP_CONTEXT:
       g_value_set_object (value, speller->priv->context);
       break;
+    case PROP_COUNT:
+      g_value_set_int (value, speller->priv->results);
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -270,49 +279,36 @@ row_activated_cb (GtkTreeView       *treeview,
   g_free (db_name);
 }
 
+static void
+clear_button_clicked_cb (GtkWidget *widget,
+			 gpointer   user_data)
+{
+  GdictSpeller *speller = GDICT_SPELLER (user_data);
+
+  gdict_speller_clear (speller);
+}
+
 static GObject *
 gdict_speller_constructor (GType                  type,
-			   guint                  n_construct_properties,
-			   GObjectConstructParam *construct_params)
+			   guint                  n_params,
+			   GObjectConstructParam *params)
 {
   GObject *object;
   GdictSpeller *speller;
   GdictSpellerPrivate *priv;
-  GtkWidget *hbox;
-  GtkWidget *label;
   GtkWidget *sw;
   GtkCellRenderer *renderer;
   GtkTreeViewColumn *column;
+  GtkWidget *hbox;
   
   object = G_OBJECT_CLASS (gdict_speller_parent_class)->constructor (type,
-  						                     n_construct_properties,
-								     construct_params);
+  						                     n_params,
+								     params);
   speller = GDICT_SPELLER (object);
   priv = speller->priv;
   
   gtk_widget_push_composite_child ();
 
-  hbox = gtk_hbox_new (FALSE, 6);
-  gtk_box_pack_start (GTK_BOX (speller), hbox, FALSE, FALSE, 0);
-  gtk_widget_show (hbox);
-
-  label = gtk_label_new (_("Similar words"));
-  gtk_widget_set_composite_name (label, "gdict-speller-label");
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-  gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
-  gtk_widget_show (label);
-
-  priv->close = gtk_button_new ();
-  gtk_widget_set_composite_name (priv->close, "gdict-speller-close-button");
-  gtk_button_set_image (GTK_BUTTON (priv->close),
-		        gtk_image_new_from_stock (GTK_STOCK_CLOSE,
-				                  GTK_ICON_SIZE_MENU));
-  gtk_button_set_relief (GTK_BUTTON (priv->close), GTK_RELIEF_NONE);
-  g_signal_connect_swapped (priv->close, "clicked",
-		            G_CALLBACK (gdict_speller_closed), speller);
-  gtk_box_pack_end (GTK_BOX (hbox), priv->close, FALSE, FALSE, 0);
-  gtk_widget_show (priv->close);
-
   sw = gtk_scrolled_window_new (NULL, NULL);
   gtk_widget_set_composite_name (sw, "gdict-speller-scrolled-window");
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
@@ -340,6 +336,24 @@ gdict_speller_constructor (GType                  type,
   gtk_container_add (GTK_CONTAINER (sw), priv->treeview);
   gtk_widget_show (priv->treeview);
 
+  hbox = gtk_hbox_new (FALSE, 0);
+
+  priv->clear_button = gtk_button_new ();
+  gtk_button_set_image (GTK_BUTTON (priv->clear_button),
+		  	gtk_image_new_from_stock (GTK_STOCK_CLEAR,
+						  GTK_ICON_SIZE_SMALL_TOOLBAR));
+  g_signal_connect (priv->clear_button, "clicked",
+		    clear_button_clicked_cb,
+		    speller);
+  gtk_box_pack_start (GTK_BOX (hbox), priv->clear_button, FALSE, FALSE, 0);
+  gtk_widget_show (priv->clear_button);
+  gtk_tooltips_set_tip (priv->tips, priv->clear_button,
+		        _("Clear the list of similar words"),
+			NULL);
+
+  gtk_box_pack_end (GTK_BOX (speller), hbox, FALSE, FALSE, 0);
+  gtk_widget_show (hbox);
+
   gtk_widget_pop_composite_child ();
 
   return object;
@@ -387,14 +401,6 @@ gdict_speller_class_init (GdictSpellerClass *klass)
 		  G_TYPE_NONE, 2,
 		  G_TYPE_STRING,
 		  G_TYPE_STRING);
-  speller_signals[CLOSED] =
-    g_signal_new ("closed",
-		  G_OBJECT_CLASS_TYPE (gobject_class),
-		  G_SIGNAL_RUN_LAST,
-		  G_STRUCT_OFFSET (GdictSpellerClass, closed),
-		  NULL, NULL,
-		  gdict_marshal_VOID__VOID,
-		  G_TYPE_NONE, 0);
   
   g_type_class_add_private (gobject_class, sizeof (GdictSpellerPrivate));
 }
@@ -418,6 +424,9 @@ gdict_speller_init (GdictSpeller *speller)
 		                    G_TYPE_STRING, /* db_name */
 				    G_TYPE_STRING  /* word */);
 
+  priv->tips = gtk_tooltips_new ();
+  g_object_ref_sink (G_OBJECT (priv->tips));
+  
   priv->start_id = 0;
   priv->end_id = 0;
   priv->match_id = 0;
@@ -716,7 +725,7 @@ gdict_speller_match (GdictSpeller *speller,
 
   if (priv->is_searching)
     {
-      _gdict_show_error_dialog (GTK_WIDGET (speller),
+      _gdict_show_error_dialog (NULL,
                                 _("Another search is in progress"),
                                 _("Please wait until the current search ends."));
 
@@ -762,7 +771,8 @@ gdict_speller_match (GdictSpeller *speller,
 			  MATCH_COLUMN_DB_NAME, _("Error while matching"),
 			  MATCH_COLUMN_WORD, NULL,
 			  -1);
-
+      
+      _gdict_debug ("Error while matching: %s", match_error->message);
       g_error_free (match_error);
     }
 }
@@ -804,19 +814,3 @@ gdict_speller_get_matches (GdictSpeller *speller,
   
   return NULL;
 }
-
-/**
- * gdict_speller_closed:
- * @speller: a #GdictSpeller
- *
- * FIXME
- *
- * Since: FIXME
- */
-void
-gdict_speller_closed (GdictSpeller *speller)
-{
-  g_return_if_fail (GDICT_IS_SPELLER (speller));
-
-  g_signal_emit (speller, speller_signals[CLOSED], 0);
-}
