@@ -40,12 +40,14 @@
 #include <libgnomeprintui/gnome-print-dialog.h>
 #include <libgnomeprintui/gnome-print-job-preview.h>
 
-#include "gdict.h"
+#include <libgdict/gdict.h>
 
+#include "gdict-sidebar.h"
 #include "gdict-print.h"
 #include "gdict-pref-dialog.h"
 #include "gdict-about.h"
 #include "gdict-window.h"
+#include "gdict-common.h"
 
 #define GDICT_WINDOW_COLUMNS      56
 #define GDICT_WINDOW_ROWS         33
@@ -53,6 +55,13 @@
 #define GDICT_WINDOW_MIN_WIDTH	  400
 #define GDICT_WINDOW_MIN_HEIGHT	  330
 
+enum
+{
+  COMPLETION_TEXT_COLUMN,
+
+  COMPLETION_N_COLUMNS
+};
+
 enum
 {
   PROP_0,
@@ -88,33 +97,6 @@ static const guint n_drop_types = G_N_ELEMENTS (drop_types);
 G_DEFINE_TYPE (GdictWindow, gdict_window, GTK_TYPE_WINDOW);
 
 
-/* shows an error dialog making it transient for @parent */
-static void
-show_error_dialog (GtkWindow   *parent,
-		   const gchar *message,
-		   const gchar *detail)
-{
-  GtkWidget *dialog;
-  
-  dialog = gtk_message_dialog_new (parent,
-  				   GTK_DIALOG_DESTROY_WITH_PARENT,
-  				   GTK_MESSAGE_ERROR,
-  				   GTK_BUTTONS_OK,
-  				   "%s", message);
-  gtk_window_set_title (GTK_WINDOW (dialog), "");
-  
-  if (detail)
-    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-  					      "%s", detail);
-  
-  if (parent->group)
-    gtk_window_group_add_window (parent->group, GTK_WINDOW (dialog));
-  
-  gtk_dialog_run (GTK_DIALOG (dialog));
-  
-  gtk_widget_destroy (dialog);
-}
-
 static void
 gdict_window_finalize (GObject *object)
 {
@@ -132,6 +114,7 @@ gdict_window_finalize (GObject *object)
   if (window->context)
     {
       g_signal_handler_disconnect (window->context, window->lookup_start_id);
+      g_signal_handler_disconnect (window->context, window->definition_id);
       g_signal_handler_disconnect (window->context, window->lookup_end_id);
       g_signal_handler_disconnect (window->context, window->error_id);
 
@@ -146,8 +129,14 @@ gdict_window_finalize (GObject *object)
   
   g_object_unref (window->action_group);
 
-  if (window->icon)
-    g_object_unref (window->icon);
+  if (window->completion)
+    g_object_unref (window->completion);
+
+  if (window->completion_model)
+    g_object_unref (window->completion_model);
+
+  if (window->busy_cursor)
+    gdk_cursor_unref (window->busy_cursor);
 
   g_free (window->source_name);
   g_free (window->print_font);
@@ -159,25 +148,89 @@ gdict_window_finalize (GObject *object)
   G_OBJECT_CLASS (gdict_window_parent_class)->finalize (object);
 }
 
+static void
+gdict_window_set_sidebar_visible (GdictWindow *window,
+				  gboolean     is_visible)
+{
+  g_assert (GDICT_IS_WINDOW (window));
+  
+  if (is_visible != window->sidebar_visible)
+    {
+      if (is_visible)
+	gtk_widget_show (window->sidebar_frame);
+      else
+	gtk_widget_hide (window->sidebar_frame);
+
+      window->sidebar_visible = is_visible;
+    }
+}
+
+static void
+gdict_window_set_statusbar_visible (GdictWindow *window,
+				    gboolean     is_visible)
+{
+  g_assert (GDICT_IS_WINDOW (window));
+
+  if (is_visible != window->statusbar_visible)
+    {
+      if (is_visible)
+	gtk_widget_show (window->status);
+      else
+	gtk_widget_hide (window->status);
+
+      window->statusbar_visible = is_visible;
+    }
+}
+
+static void
+gdict_window_definition_cb (GdictContext    *context,
+			    GdictDefinition *definition,
+			    GdictWindow     *window)
+{
+  gint total, n;
+  gdouble fraction;
+
+  g_assert (GDICT_IS_WINDOW (window));
+
+  total = gdict_definition_get_total (definition);
+  n = window->current_definition + 1;
+
+  fraction = CLAMP (((gdouble) n / (gdouble) total), 0.0, 1.0);
+
+  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (window->progress),
+		  		 fraction);
+  while (gtk_events_pending ())
+    gtk_main_iteration ();
+
+  window->current_definition = n;
+}
+
 static void
 gdict_window_lookup_start_cb (GdictContext *context,
 			      GdictWindow  *window)
 {
-  static GdkCursor *busy_cursor = NULL;
   gchar *message;
 
   if (!window->word)
     return;
 
-  if (busy_cursor == NULL)
-    busy_cursor = gdk_cursor_new (GDK_WATCH);
+  if (!window->busy_cursor)
+    window->busy_cursor = gdk_cursor_new (GDK_WATCH);
 
   message = g_strdup_printf (_("Searching for '%s'..."), window->word);
   
   if (window->status)
     gtk_statusbar_push (GTK_STATUSBAR (window->status), 0, message);
 
-  gdk_window_set_cursor (GTK_WIDGET (window)->window, busy_cursor);
+  if (window->progress)
+    gtk_widget_show (window->progress);
+
+  window->max_definition = -1;
+  window->last_definition = 0;
+  window->current_definition = 0;
+
+  gdk_window_set_cursor (GTK_WIDGET (window)->window,
+		         window->busy_cursor);
   g_free (message);
 }
 
@@ -187,15 +240,13 @@ gdict_window_lookup_end_cb (GdictContext *context,
 {
   gchar *message;
   gint count;
+  GtkTreeIter iter;
 
-  count = gdict_defbox_count_definitions (GDICT_DEFBOX (window->defbox));
+  count = window->current_definition;
 
-  if (count == -1)
-    window->max_definition = -1;
-  else
-    window->max_definition = count;
+  window->max_definition = count - 1;
 
-  if (count == -1)
+  if (count == 0)
     message = g_strdup (_("No definitions found"));
   else 
     message = g_strdup_printf (ngettext("A definition found",
@@ -206,6 +257,14 @@ gdict_window_lookup_end_cb (GdictContext *context,
   if (window->status)
     gtk_statusbar_push (GTK_STATUSBAR (window->status), 0, message);
 
+  if (window->progress)
+    gtk_widget_hide (window->progress);
+
+  gtk_list_store_append (window->completion_model, &iter);
+  gtk_list_store_set (window->completion_model, &iter,
+		      COMPLETION_TEXT_COLUMN, window->word,
+		      -1);
+
   gdk_window_set_cursor (GTK_WIDGET (window)->window, NULL);
   g_free (message);
 }
@@ -215,10 +274,43 @@ gdict_window_error_cb (GdictContext *context,
 		       const GError *error,
 		       GdictWindow  *window)
 {
+  gint count;
+  
   gdk_window_set_cursor (GTK_WIDGET (window)->window, NULL);
-
+  
   gtk_statusbar_push (GTK_STATUSBAR (window->status), 0,
 		      _("No definitions found"));
+
+  gtk_widget_hide (window->progress);
+
+  /* launch the speller only on NO_MATCH */
+  if (error->code == GDICT_CONTEXT_ERROR_NO_MATCH)
+    {
+      GdictSource *source;
+      GdictContext *context;
+
+      gdict_window_set_sidebar_visible (window, TRUE);
+      gdict_sidebar_view_page (GDICT_SIDEBAR (window->sidebar), "speller");
+
+      /* we clone the context, so that the signals that it
+       * fires do not get caught by the signal handlers we
+       * use for getting the definitions.
+       */
+      source = gdict_source_loader_get_source (window->loader,
+		      			       window->source_name);
+      context = gdict_source_get_context (source);
+
+      gdict_speller_set_context (GDICT_SPELLER (window->speller),
+		      		 context);
+      g_object_unref (context);
+      g_object_unref (source);
+      
+      gdict_speller_set_strategy (GDICT_SPELLER (window->speller),
+			          window->strategy);
+      
+      gdict_speller_match (GDICT_SPELLER (window->speller),
+		           window->word);
+    }
 }
 
 static void
@@ -270,9 +362,9 @@ get_context_from_loader (GdictWindow *window)
       detail = g_strdup_printf (_("No dictionary source available with name '%s'"),
       				window->source_name);
 
-      show_error_dialog (GTK_WINDOW (window),
-                         _("Unable to find dictionary source"),
-                         NULL);
+      gdict_show_error_dialog (GTK_WINDOW (window),
+                               _("Unable to find dictionary source"),
+                               NULL);
       
       g_free (detail);
 
@@ -290,9 +382,9 @@ get_context_from_loader (GdictWindow *window)
       detail = g_strdup_printf (_("No context available for source '%s'"),
       				gdict_source_get_description (source));
       				
-      show_error_dialog (GTK_WINDOW (window),
-                         _("Unable to create a context"),
-                         detail);
+      gdict_show_error_dialog (GTK_WINDOW (window),
+                               _("Unable to create a context"),
+                               detail);
       
       g_free (detail);
       g_object_unref (source);
@@ -349,7 +441,8 @@ gdict_window_set_print_font (GdictWindow *window,
 
 static void
 gdict_window_set_word (GdictWindow *window,
-		       const gchar *word)
+		       const gchar *word,
+		       const gchar *database)
 {
   gchar *title;
   
@@ -358,6 +451,11 @@ gdict_window_set_word (GdictWindow *window,
 
   if (word && word[0] != '\0')
     window->word = g_strdup (word);
+  else
+    return;
+
+  if (!database || database[0] == '\0')
+    database = window->database;
 
   if (window->word)
     title = g_strdup_printf (_("%s - Dictionary"), window->word);
@@ -366,12 +464,12 @@ gdict_window_set_word (GdictWindow *window,
   
   gtk_window_set_title (GTK_WINDOW (window), title);
   g_free (title);
-  
-  window->max_definition = -1;
-  window->last_definition = 0;
-  
+
   if (window->defbox)
-    gdict_defbox_lookup (GDICT_DEFBOX (window->defbox), window->word);
+    {
+      gdict_defbox_set_database (GDICT_DEFBOX (window->defbox), database);
+      gdict_defbox_lookup (GDICT_DEFBOX (window->defbox), word);
+    }
 }
 
 static void
@@ -380,10 +478,12 @@ gdict_window_set_context (GdictWindow  *window,
 {
   if (window->context)
     {
+      g_signal_handler_disconnect (window->context, window->definition_id);
       g_signal_handler_disconnect (window->context, window->lookup_start_id);
       g_signal_handler_disconnect (window->context, window->lookup_end_id);
       g_signal_handler_disconnect (window->context, window->error_id);
       
+      window->definition_id = 0;
       window->lookup_start_id = 0;
       window->lookup_end_id = 0;
       window->error_id = 0;
@@ -399,6 +499,9 @@ gdict_window_set_context (GdictWindow  *window,
     return;
   
   /* attach our callbacks */
+  window->definition_id   = g_signal_connect (context, "definition-found",
+		  			      G_CALLBACK (gdict_window_definition_cb),
+					      window);
   window->lookup_start_id = g_signal_connect (context, "lookup-start",
 		  			      G_CALLBACK (gdict_window_lookup_start_cb),
 					      window);
@@ -457,7 +560,7 @@ gdict_window_set_property (GObject      *object,
       gdict_window_set_source_name (window, g_value_get_string (value));
       break;
     case PROP_WORD:
-      gdict_window_set_word (window, g_value_get_string (value));
+      gdict_window_set_word (window, g_value_get_string (value), NULL);
       break;
     case PROP_PRINT_FONT:
       gdict_window_set_print_font (window, g_value_get_string (value));
@@ -526,6 +629,22 @@ gdict_window_cmd_file_new (GtkAction   *action,
 		  	 GDICT_GCONF_WINDOW_IS_MAXIMIZED_KEY,
 			 window->is_maximized,
 			 NULL);
+  gconf_client_set_bool (window->gconf_client,
+		  	 GDICT_GCONF_SIDEBAR_VISIBLE_KEY,
+			 window->sidebar_visible,
+			 NULL);
+  gconf_client_set_int (window->gconf_client,
+		  	GDICT_GCONF_SIDEBAR_WIDTH_KEY,
+			window->sidebar_width,
+			NULL);
+  gconf_client_set_string (window->gconf_client,
+		  	   GDICT_GCONF_SIDEBAR_PAGE_KEY,
+			   gdict_sidebar_current_page (GDICT_SIDEBAR (window->sidebar)),
+			   NULL);
+  gconf_client_set_bool (window->gconf_client,
+		  	 GDICT_GCONF_STATUSBAR_VISIBLE_KEY,
+			 window->statusbar_visible,
+			 NULL);
  
   new_window = gdict_window_new (window->loader, NULL, NULL);
   gtk_widget_show (new_window);
@@ -574,11 +693,10 @@ gdict_window_cmd_save_as (GtkAction   *action,
           
           message = g_strdup_printf (_("Error while writing to '%s'"), filename);
           
-          show_error_dialog (GTK_WINDOW (window),
-          		     message,
-          		     write_error->message);
-          
-          g_error_free (write_error);
+          gdict_show_gerror_dialog (GTK_WINDOW (window),
+                                    message,
+                                    write_error);
+
           g_free (message);
         }
       
@@ -619,10 +737,26 @@ gdict_window_cmd_file_close_window (GtkAction   *action,
 		  	 GDICT_GCONF_WINDOW_IS_MAXIMIZED_KEY,
 			 window->is_maximized,
 			 NULL);
+  gconf_client_set_bool (window->gconf_client,
+		         GDICT_GCONF_SIDEBAR_VISIBLE_KEY,
+			 window->sidebar_visible,
+			 NULL);
+  gconf_client_set_int  (window->gconf_client,
+		  	 GDICT_GCONF_SIDEBAR_WIDTH_KEY,
+			 window->sidebar_width,
+			 NULL);
+  gconf_client_set_string (window->gconf_client,
+		           GDICT_GCONF_SIDEBAR_PAGE_KEY,
+			   gdict_sidebar_current_page (GDICT_SIDEBAR (window->sidebar)),
+			   NULL);
+  gconf_client_set_bool (window->gconf_client,
+		  	 GDICT_GCONF_STATUSBAR_VISIBLE_KEY,
+			 window->statusbar_visible,
+			 NULL);
   
   /* if this was called from the uimanager, destroy the widget;
-   * otherwise, if it was called from the delete_event, it will
-   * destroy the widget itself.
+   * otherwise, if it was called from the delete_event, the widget
+   * will destroy itself.
    */
   if (action)
     gtk_widget_destroy (GTK_WIDGET (window));
@@ -678,15 +812,57 @@ static void
 gdict_window_cmd_edit_preferences (GtkAction   *action,
 				   GdictWindow *window)
 {
-  GtkWidget *pref_dialog;
+  g_assert (GDICT_IS_WINDOW (window));
   
-  pref_dialog = gdict_pref_dialog_new (GTK_WINDOW (window),
-  				       _("Dictionary Preferences"),
-  				       window->loader);
+  gdict_show_pref_dialog (GTK_WINDOW (window),
+  			  _("Dictionary Preferences"),
+  			  window->loader);
+}
+
+static void
+gdict_window_cmd_view_sidebar (GtkAction   *action,
+			       GdictWindow *window)
+{
+  gboolean is_active;
   
-  gtk_dialog_run (GTK_DIALOG (pref_dialog));
-  
-  gtk_widget_destroy (pref_dialog);
+  g_assert (GDICT_IS_WINDOW (window));
+
+  if (window->sidebar_visible)
+    gdict_window_set_sidebar_visible (window, FALSE);
+  else
+    gdict_window_set_sidebar_visible (window, TRUE);
+}
+
+static void
+gdict_window_cmd_view_statusbar (GtkAction   *action,
+				 GdictWindow *window)
+{
+  g_assert (GDICT_IS_WINDOW (window));
+
+  if (window->statusbar_visible)
+    gdict_window_set_statusbar_visible (window, FALSE);
+  else
+    gdict_window_set_statusbar_visible (window, TRUE);
+}
+
+static void
+gdict_window_cmd_view_speller (GtkAction   *action,
+			       GdictWindow *window)
+{
+  g_assert (GDICT_IS_WINDOW (window));
+
+  gdict_sidebar_view_page (GDICT_SIDEBAR (window->sidebar), "speller");
+  gdict_window_set_sidebar_visible (window, TRUE);
+}
+
+static void
+gdict_window_cmd_view_databases (GtkAction   *action,
+				 GdictWindow *window)
+{
+  g_assert (GDICT_IS_WINDOW (window));
+
+  gdict_sidebar_view_page (GDICT_SIDEBAR (window->sidebar), "databases");
+  gdict_window_set_sidebar_visible (window, TRUE);
 }
 
 static void
@@ -761,10 +937,9 @@ gdict_window_cmd_help_contents (GtkAction   *action,
   					&err);
   if (err)
     {
-      show_error_dialog (GTK_WINDOW (window),
-      			 _("There was an error while displaying help"),
-      			 err->message);
-      g_error_free (err);
+      gdict_show_gerror_dialog (GTK_WINDOW (window),
+      		                _("There was an error while displaying help"),
+      			        err);
     }
 }
 
@@ -799,6 +974,7 @@ static const GtkActionEntry entries[] =
 {
   { "File", NULL, N_("_File") },
   { "Edit", NULL, N_("_Edit") },
+  { "View", NULL, N_("_View") },
   { "Go", NULL, N_("_Go") },
   { "Help", NULL, N_("_Help") },
 
@@ -827,6 +1003,16 @@ static const GtkActionEntry entries[] =
   { "EditPreferences", GTK_STOCK_PREFERENCES, N_("_Preferences"), NULL, NULL,
     G_CALLBACK (gdict_window_cmd_edit_preferences) },
 
+  /* View menu */
+  { "ViewSidebar", NULL, N_("_Sidebar"), "F9", NULL,
+    G_CALLBACK (gdict_window_cmd_view_sidebar) },
+  { "ViewStatusbar", NULL, N_("S_tatusbar"), NULL, NULL,
+    G_CALLBACK (gdict_window_cmd_view_statusbar) },
+  { "ViewSpeller", NULL, N_("Similar _Words"), "<control>T", NULL,
+    G_CALLBACK (gdict_window_cmd_view_speller) },
+  { "ViewDB", NULL, N_("Available_Databases"), "<control>B", NULL,
+    G_CALLBACK (gdict_window_cmd_view_databases) },
+
   /* Go menu */
   { "GoPreviousDef", GTK_STOCK_GO_BACK, N_("_Previous Definition"), "<control>Page_Up",
     N_("Go to the previous definition"), G_CALLBACK (gdict_window_cmd_go_previous_def) },
@@ -843,7 +1029,7 @@ static const GtkActionEntry entries[] =
   { "HelpAbout", GTK_STOCK_ABOUT, N_("_About"), NULL, NULL,
     G_CALLBACK (gdict_window_cmd_help_about) },
   
-  /* Accellerators */
+  /* Accelerators */
   { "Lookup", NULL, "", "<control>L", NULL, G_CALLBACK (gdict_window_cmd_lookup) },
   { "Escape", NULL, "", "Escape", "", G_CALLBACK (gdict_window_cmd_escape) },
   { "Slash", GTK_STOCK_FIND, NULL, "slash", NULL, G_CALLBACK (gdict_window_cmd_edit_find) },
@@ -912,6 +1098,22 @@ gdict_window_gconf_notify_cb (GConfClient *client,
       else
         gdict_window_set_defbox_font (window, GDICT_DEFAULT_DEFBOX_FONT);
     }
+  else if (strcmp (entry->key, GDICT_GCONF_SIDEBAR_VISIBLE_KEY) == 0)
+    {
+      if (entry->value && (entry->value->type == GCONF_VALUE_BOOL))
+        gdict_window_set_sidebar_visible (window,
+					  gconf_value_get_bool (entry->value));
+      else
+        gdict_window_set_sidebar_visible (window, FALSE);
+    }
+  else if (strcmp (entry->key, GDICT_GCONF_STATUSBAR_VISIBLE_KEY) == 0)
+    {
+      if (entry->value && (entry->value->type == GCONF_VALUE_BOOL))
+	gdict_window_set_statusbar_visible (window,
+					    gconf_value_get_bool (entry->value));
+      else
+	gdict_window_set_statusbar_visible (window, FALSE);
+    }
 }
 
 static void
@@ -930,7 +1132,79 @@ entry_activate_cb (GtkWidget   *widget,
   if (!word || *word == '\0')
     return;
 
-  gdict_window_set_word (window, word);
+  gdict_window_set_word (window, word, NULL);
+}
+
+static void
+database_activated_cb (GdictDatabaseChooser *chooser,
+		       const gchar          *db_name,
+		       const gchar          *db_desc,
+		       GdictWindow          *window)
+{
+  gdict_window_set_database (window, db_name);
+
+  if (window->status)
+    {
+      gchar *message;
+
+      message = g_strdup_printf (_("Database `%s' selected"), db_desc);
+      gtk_statusbar_push (GTK_STATUSBAR (window->status), 0, message);
+      g_free (message);
+    }
+}
+
+static void
+speller_word_activated_cb (GdictSpeller *speller,
+			   const gchar  *word,
+			   const gchar  *db_name,
+			   GdictWindow  *window)
+{
+  gtk_entry_set_text (GTK_ENTRY (window->entry), word);
+  
+  gdict_window_set_word (window, word, db_name);
+
+  if (window->status)
+    {
+      gchar *message;
+
+      message = g_strdup_printf (_("Word `%s' selected"), word);
+      gtk_statusbar_push (GTK_STATUSBAR (window->status), 0, message);
+      g_free (message);
+    }
+}
+
+static void
+sidebar_page_changed_cb (GdictSidebar *sidebar,
+			 GdictWindow  *window)
+{
+  const gchar *page_id;
+  const gchar *message;
+
+  page_id = gdict_sidebar_current_page (sidebar);
+
+  switch (page_id[0])
+    {
+    case 's': /* speller */
+      message = _("Double-click on the word to look up");
+      break;
+    case 'd': /* db-chooser */
+      message = _("Double-click on the database to use");
+      break;
+    default:
+      message = NULL;
+      break;
+    }
+
+  if (message && window->status)
+    gtk_statusbar_push (GTK_STATUSBAR (window->status), 0, message);
+}
+
+static void
+sidebar_closed_cb (GdictSidebar *sidebar,
+		   GdictWindow  *window)
+{
+  gtk_widget_hide (window->sidebar_frame);
+  window->sidebar_visible = FALSE;
 }
 
 static void
@@ -953,7 +1227,7 @@ gdict_window_drag_data_received_cb (GtkWidget        *widget,
       
       gtk_entry_set_text (GTK_ENTRY (window->entry), text);
 
-      gdict_window_set_word (window, text);
+      gdict_window_set_word (window, text, NULL);
       g_free (text);
       
       gtk_drag_finish (context, TRUE, FALSE, time_);
@@ -1062,6 +1336,20 @@ gdict_window_style_set (GtkWidget *widget,
   set_window_default_size (GDICT_WINDOW (widget));
 }
 
+static void
+gdict_window_handle_notify_position_cb (GtkWidget  *widget,
+					GParamSpec *pspec,
+					gpointer    user_data)
+{
+  GdictWindow *window = GDICT_WINDOW (user_data);
+  gint window_width, pos;
+
+  pos = gtk_paned_get_position (GTK_PANED (widget));
+  window_width = GTK_WIDGET (window)->allocation.width;
+
+  window->sidebar_width = window_width - pos;
+}
+
 static GObject *
 gdict_window_constructor (GType                  type,
 			  guint                  n_construct_properties,
@@ -1069,16 +1357,21 @@ gdict_window_constructor (GType                  type,
 {
   GObject *object;
   GdictWindow *window;
-  gint width, height;
+  gint width, height, sidebar_width;
   gboolean is_maximized;
   GtkWidget *hbox;
+  GtkWidget *handle;
+  GtkWidget *frame1, *frame2;
   GtkWidget *vbox;
   GtkWidget *label;
   GtkActionGroup *action_group;
   GtkAccelGroup *accel_group;
+  GtkAction *action;
   PangoFontDescription *font_desc;
-  gchar *font_name;
+  gchar *font_name, *sidebar_page;
   GError *error;
+  gboolean sidebar_visible;
+  gboolean statusbar_visible;
   
   object = G_OBJECT_CLASS (gdict_window_parent_class)->constructor (type,
   						   n_construct_properties,
@@ -1133,17 +1426,36 @@ gdict_window_constructor (GType                  type,
   label = gtk_label_new_with_mnemonic (_("Look _up:"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
+
+  window->completion_model = gtk_list_store_new (COMPLETION_N_COLUMNS,
+		  				 G_TYPE_STRING);
+  
+  window->completion = gtk_entry_completion_new ();
+  gtk_entry_completion_set_popup_completion (window->completion, TRUE);
+  gtk_entry_completion_set_model (window->completion,
+		  		  GTK_TREE_MODEL (window->completion_model));
+  gtk_entry_completion_set_text_column (window->completion,
+		  			COMPLETION_TEXT_COLUMN);
   
   window->entry = gtk_entry_new ();
   if (window->word)
     gtk_entry_set_text (GTK_ENTRY (window->entry), window->word);
   
+  gtk_entry_set_completion (GTK_ENTRY (window->entry),
+		  	    window->completion);
   g_signal_connect (window->entry, "activate", G_CALLBACK (entry_activate_cb), window);
   gtk_box_pack_start (GTK_BOX (hbox), window->entry, TRUE, TRUE, 0);
   gtk_widget_show (window->entry);
 
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), window->entry);
 
+  handle = gtk_hpaned_new ();
+  gtk_box_pack_start (GTK_BOX (vbox), handle, TRUE, TRUE, 0);
+  gtk_widget_show (handle);
+
+  frame1 = gtk_vbox_new (FALSE, 0);
+  frame2 = gtk_vbox_new (FALSE, 0);
+  
   window->defbox = gdict_defbox_new ();
   if (window->context)
     gdict_defbox_set_context (GDICT_DEFBOX (window->defbox), window->context);
@@ -1155,13 +1467,65 @@ gdict_window_constructor (GType                  type,
   g_signal_connect (window->defbox, "drag-data-received",
   		    G_CALLBACK (gdict_window_drag_data_received_cb),
   		    window);
-  gtk_box_pack_start (GTK_BOX (vbox), window->defbox, TRUE, TRUE, 0);
-  gtk_widget_show_all (window->defbox);
+  gtk_container_add (GTK_CONTAINER (frame1), window->defbox);
+  gtk_widget_show (window->defbox);
+
+  window->sidebar = gdict_sidebar_new ();
+  g_signal_connect (window->sidebar, "page-changed",
+		    G_CALLBACK (sidebar_page_changed_cb),
+		    window);
+  g_signal_connect (window->sidebar, "closed",
+		    G_CALLBACK (sidebar_closed_cb),
+		    window);
+  
+  window->speller = gdict_speller_new ();
+  if (window->context)
+    gdict_speller_set_context (GDICT_SPELLER (window->speller),
+		    	       window->context);
+  g_signal_connect (window->speller, "word-activated",
+		    G_CALLBACK (speller_word_activated_cb),
+		    window);
+  
+  gdict_sidebar_add_page (GDICT_SIDEBAR (window->sidebar),
+		          "speller",
+			  _("Similar words"),
+			  window->speller);
+  gtk_widget_show (window->speller);
+
+  window->db_chooser = gdict_database_chooser_new ();
+  if (window->context)
+    gdict_database_chooser_set_context (GDICT_DATABASE_CHOOSER (window->db_chooser),
+			    		window->context);
+  g_signal_connect (window->db_chooser, "database-activated",
+	  	    G_CALLBACK (database_activated_cb),
+		    window);
+  gdict_sidebar_add_page (GDICT_SIDEBAR (window->sidebar),
+	  		  "db-chooser",
+			  _("Available dictionaries"),
+			  window->db_chooser);
+  gtk_widget_show (window->db_chooser);
+  
+  gtk_container_add (GTK_CONTAINER (frame2), window->sidebar);
+  gtk_widget_show (window->sidebar);
+
+  gtk_paned_pack1 (GTK_PANED (handle), frame1, TRUE, FALSE);
+  gtk_paned_pack2 (GTK_PANED (handle), frame2, FALSE, TRUE);
+
+  window->defbox_frame = frame1;
+  window->sidebar_frame = frame2;
+
+  gtk_widget_show (window->defbox_frame);
 
   window->status = gtk_statusbar_new ();
   gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (window->status), TRUE);
   gtk_box_pack_end (GTK_BOX (window->main_box), window->status, FALSE, FALSE, 0);
-  gtk_widget_show (window->status);
+  statusbar_visible = gconf_client_get_bool (window->gconf_client,
+		  			     GDICT_GCONF_STATUSBAR_VISIBLE_KEY,
+					     NULL);
+  gdict_window_set_statusbar_visible (window, statusbar_visible);
+
+  window->progress = gtk_progress_bar_new ();
+  gtk_box_pack_end (GTK_BOX (window->status), window->progress, FALSE, FALSE, 0);
 
   /* retrieve the font size from gconf */
   font_name = gconf_client_get_string (window->gconf_client,
@@ -1172,7 +1536,12 @@ gdict_window_constructor (GType                  type,
   
   gdict_window_set_defbox_font (window, font_name);
   font_desc = pango_font_description_from_string (font_name);
-  
+
+  sidebar_visible = gconf_client_get_bool (window->gconf_client,
+		  			   GDICT_GCONF_SIDEBAR_VISIBLE_KEY,
+					   NULL);
+  gdict_window_set_sidebar_visible (window, sidebar_visible);
+
   /* retrieve the window state from gconf */
   is_maximized = gconf_client_get_bool (window->gconf_client,
 		  			GDICT_GCONF_WINDOW_IS_MAXIMIZED_KEY,
@@ -1184,7 +1553,13 @@ gdict_window_constructor (GType                  type,
   height = gconf_client_get_int (window->gconf_client,
 		  		 GDICT_GCONF_WINDOW_HEIGHT_KEY,
 				 NULL);
-  
+  sidebar_width = gconf_client_get_int (window->gconf_client,
+		  			GDICT_GCONF_SIDEBAR_WIDTH_KEY,
+					NULL);
+  sidebar_page = gconf_client_get_string (window->gconf_client,
+		  			  GDICT_GCONF_SIDEBAR_PAGE_KEY,
+					  NULL);
+
   /* if the (width, height) tuple is not defined, use the font to
    * calculate the right window geometry
    */
@@ -1214,6 +1589,10 @@ gdict_window_constructor (GType                  type,
   			       height);
   if (is_maximized)
     gtk_window_maximize (GTK_WINDOW (window));
+
+  gtk_paned_set_position (GTK_PANED (handle),
+		          GTK_WIDGET (window)->allocation.width - sidebar_width);
+  gdict_sidebar_view_page (GDICT_SIDEBAR (window->sidebar), sidebar_page);
   
   g_signal_connect (window, "delete-event",
 		    G_CALLBACK (gdict_window_delete_event_cb),
@@ -1221,6 +1600,9 @@ gdict_window_constructor (GType                  type,
   g_signal_connect (window, "window-state-event",
 		    G_CALLBACK (gdict_window_state_event_cb),
 		    NULL);
+  g_signal_connect (handle, "notify::position",
+		    G_CALLBACK (gdict_window_handle_notify_position_cb),
+		    window);
 
   gtk_widget_pop_composite_child ();
   
@@ -1300,9 +1682,7 @@ gdict_window_class_init (GdictWindowClass *klass)
 static void
 gdict_window_init (GdictWindow *window)
 {
-  gchar *icon_file;
-  GdkPixbuf *icon;
-  GError *icon_error, *gconf_error;
+  GError *gconf_error;
   
   window->loader = NULL;
   window->context = NULL;
@@ -1316,12 +1696,9 @@ gdict_window_init (GdictWindow *window)
   			&gconf_error);
   if (gconf_error)
     {
-      show_error_dialog (NULL,
-		         _("Unable to connect to GConf"),
-		         gconf_error->message);
-
-      g_error_free (gconf_error);
-      gconf_error = NULL;
+      gdict_show_gerror_dialog (NULL,
+		                _("Unable to connect to GConf"),
+		                gconf_error);
     }
 
   window->notify_id = gconf_client_notify_add (window->gconf_client,
@@ -1332,12 +1709,9 @@ gdict_window_init (GdictWindow *window)
   					       &gconf_error);
   if (gconf_error)
     {
-      show_error_dialog (NULL,
-		         _("Unable to get notification for preferences"),
-		         gconf_error->message);
-
-      g_error_free (gconf_error);
-      gconf_error = NULL;
+      gdict_show_gerror_dialog (NULL,
+		                _("Unable to get notification for preferences"),
+		                gconf_error);
     }
 
   window->font_notify_id = gconf_client_notify_add (window->gconf_client,
@@ -1348,12 +1722,9 @@ gdict_window_init (GdictWindow *window)
 						    &gconf_error);
   if (gconf_error)
     {
-      show_error_dialog (NULL,
-		         _("Unable to get notification for the document font"),
-		         gconf_error->message);
-
-      g_error_free (gconf_error);
-      gconf_error = NULL;
+      gdict_show_gerror_dialog (NULL,
+		                _("Unable to get notification for the document font"),
+		                gconf_error);
     }
   
   window->word = NULL;
@@ -1364,28 +1735,6 @@ gdict_window_init (GdictWindow *window)
   window->database = NULL;
   window->strategy = NULL;
 
-  icon_file = g_build_filename (DATADIR,
-		  		"pixmaps",
-				"gnome-dictionary.png",
-				NULL);
-  icon_error = NULL;
-  icon = gdk_pixbuf_new_from_file (icon_file, &icon_error);
-  if (icon_error)
-    {
-      show_error_dialog (NULL,
-		         _("Unable to load the application icon"),
-		         icon_error->message);
-
-      g_error_free (icon_error);
-    }
-  else
-    {
-      gtk_window_set_default_icon (icon);
-      window->icon = icon;
-    }
-
-  g_free (icon_file);
-
   window->default_width = -1;
   window->default_height = -1;
   window->is_maximized = FALSE;
@@ -1411,7 +1760,7 @@ gdict_window_new (GdictSourceLoader *loader,
     {
       gtk_entry_set_text (GTK_ENTRY (GDICT_WINDOW (retval)->entry), word);
       
-      gdict_window_set_word (GDICT_WINDOW (retval), word);
+      gdict_window_set_word (GDICT_WINDOW (retval), word, NULL);
     }
 
   return retval;
